<?php
// Version 1.0
//
// Script Deteccion Vulnerabilidad Apache
// Licencia: GNU General Public License (GPL)
// Fuente: https://www.interactivetools.com/apache-cve-2019-0211-vulnerability-detection-script/
//
// By interactivetools.com - We provide expert programming services for web agencies
// ------------------------------------------------------------------------------------------------------
  
  // show all errors
  error_reporting(-1);
  ini_set('display_errors', '1');         //
  ini_set('display_startup_errors', '1'); //
  
  getApacheVersion_showPage();
  exit;

//
function getApacheVersion_showPage() {
  
  // check versions
  $versionList  = ''; 
  $versionList .= _getApacheVersion_fromBinaries();
  $versionList .= _getApacheVersion_fromPHP();
  $versionList .= _getApacheVersion_fromOther();

  //
  $apacheVersion = '';
  if (preg_match("/^([\d\.]+)/", $versionList, $matches)) {
    $apacheVersion = $matches[1];
  }
  
  // check for alternate patched versions
  $alternatePatch = false;
  if (preg_match('/2.4.7-1ubuntu4.21|2.4.18-2ubuntu3.10|2.4.29-1ubuntu4.6|2.4.34-1ubuntu2.1|2.4.38-2ubuntu2|2.4.25-3\+deb9u7/', $versionList)) {
    $alternatePatch = true;
  }

  $isVulnerable = $apacheVersion && version_compare($apacheVersion, '2.4.17', '>=') && version_compare($apacheVersion, '2.4.38', '<=');
?>
<!DOCTYPE html>
<html lang="en">
<head>
  <meta charset="UTF-8">
  <meta name="viewport" content="width=device-width, initial-scale=1.0">
  <meta http-equiv="X-UA-Compatible" content="ie=edge">
  <meta name="robots" content="noindex,nofollow">
  <title>Apache CVE-2019-0211 Deteccion Vulnerabilidad</title>
  <link rel="stylesheet" href="https://stackpath.bootstrapcdn.com/bootstrap/4.3.1/css/bootstrap.min.css" integrity="sha384-ggOyR0iXCbMQv3Xipma34MD+dH/1fQ784/j6cY/iJTQUOhcWr7x9JvoRxT2MZw1T" crossorigin="anonymous">
  <link rel="stylesheet" href="https://use.fontawesome.com/releases/v5.8.1/css/all.css" integrity="sha384-50oBUHEmvpQ+1lW4y57PTFmhCaXp0ML5d60M1M7uH2+nqUivzIebhndOJK28anvf" crossorigin="anonymous">
  
  <script src="https://code.jquery.com/jquery-3.3.1.slim.min.js" integrity="sha384-q8i/X+965DzO0rT7abK41JStQIAqVgRVzpbzo5smXKp4YfRvH+8abtTE1Pi6jizo" crossorigin="anonymous"></script>
  <script src="https://cdnjs.cloudflare.com/ajax/libs/popper.js/1.14.7/umd/popper.min.js" integrity="sha384-UO2eT0CpHqdSJQ6hJty5KVphtPhzWj9WO1clHTMGa3JDZwrnQq4sF86dIHNDz0W1" crossorigin="anonymous"></script>
  <script src="https://stackpath.bootstrapcdn.com/bootstrap/4.3.1/js/bootstrap.min.js" integrity="sha384-JjSmVgyd0p3pXB1rRibZUAYoIIy6OrQ6VrjIEaFf/nJGzIxFDsf4x0xIM+B07jRM" crossorigin="anonymous"></script>
</head>
<body>
  
  <div class="container">

    <div class="row">
      <div class="col">

        <h1 class="mt-4">Apache CVE-2019-0211 Deteccion Vulnerabilidad</h1>
        <hr>

<?php if (!$apacheVersion): ?>
        
          <p class="alert alert-warning">
            <strong><i class="far fa-question-circle"></i> Versión no determinada.</strong><br>
            La configuración de tu servidor nos impide determinar la versión de Apache.
          </p>

<?php elseif (!$isVulnerable || isWindows() || $alternatePatch): ?>
        
          <p class="alert alert-success">
            <strong><i class="far fa-check-circle"></i> Estás seguro!</strong><br>
           No estás en riesgo por ésta vulnerabilidad. Ninguna acción necesaria!
          </p>

<?php else: ?>
          <p class="alert alert-danger">
            <strong><i class="fas fa-exclamation-circle"></i> Puedes ser vulnerable!</strong><br>
            Debes contactar a tu proveedor de hospedaje para solicitar el parche de CVE-2019-0211.
          </p>
<?php endif; ?>

      </div>
    </div>
    <div class="row">
      
      <div class="col-sm-7">

        <p>El 1° de Abril de 2019 fue descubierto que cerca de 2 millones de servidores Apache eran vulnerables a un exploit
        de seguridad conocido como <strong><a href="https://nvd.nist.gov/vuln/detail/CVE-2019-0211">CVE-2019-0211</a></strong>.
        Ésta vulnerabilidad permite a cualquiera con una cuenta en el servidor a potencialmente escalar de privilegios y ganar
        control total del servidor. Los entornos de Hosting Compartido son los principales en riesgo ya que cualquiera con un acceso
        podría potencialmente obtener control completo del servidor para accesar y manipular los sitios web de otros usuarios. 
        La vulnerabilidad existe en versiones <strong>2.4.17</strong> a <strong>2.4.38</strong> del Servidor Apache HTTP y fue 
        solucionado en la versión <strong>2.4.39</strong>.</p>
      </div>
      
      <div class="col-sm-5">

      <?php if ($apacheVersion): ?>
        <div class="card">
          <div class="card-body">
            <p>Tu versión de Apache es: <b>v<?php echo $apacheVersion; ?><?php if($isVulnerable && (isWindows() || $alternatePatch)): ?>*<?php endif; ?></b> como lo reporta:<br/>
            <pre><?php echo $versionList ?></pre>
            
            <?php if($isVulnerable && isWindows()): ?>
            <p class="small">
              * Ésta versión de apache está listada como vulnerable, aunque hemos detectado que utilizas Windows server, el cual no se ve afectado.
            </p>
            <?php endif; ?>
            
            <?php if($isVulnerable && $alternatePatch): ?>
            <p class="small">
              * Ésta versión de apache está listada como vulnerable, aunque hemos detectado un binario parchado para el servidor.
            </p>
            <?php endif; ?>
          </div>
        </div>
      <?php endif ?>

      </div>
    </div>
    
    <div class="row">
      <div class="col">
        <p>Éste script fue desarrollado por <a href="https://www.mecanicaip.com/" target="_blank">MecanicaIP.com</a>.
          Ofrecemos servicios de programación para agencias.</p>
        <hr>

        <p>
          <a data-toggle="collapse" href="#collapseDebug" role="button" aria-expanded="false" aria-controls="collapseDebug">
            Mostrar información <i class="fas fa-chevron-down"></i>
          </a>
        </p>
        <div class="collapse" id="collapseDebug">
          <pre style="background-color: #EEE; padding: 10px 15px; border: 2px inset #CCC; font-family: monospace; olor: #666"><?php echo trim(_debugOutput()); ?></pre>
        </div>

      </div>
    </div>
  </div>

</body>
</html>
<?php
exit;

}
  
//
function _getApacheVersion_fromBinaries() {
  $versionList = '';

  // httpd -v
  $version = ''; 
  //if (isWindows()) { $path = "C:/wamp64/bin/apache/apache2.4.27/bin/"; }
  if (isWindows()) { $path = ""; }
  else             { $path = ""; }
  $command = "{$path}httpd -v"; //
  $output  = _getCommandOutput($command);
  if (preg_match("|Apache/(\S+)|", $output, $matches)) { // eg: Server version: Apache/2.4.6 (CentOS)
    $version = $matches[1];
    $versionList .= "$version (Source: $command)\n";
  }

  // apache2 -v
  $version = ''; 
  //if (isWindows()) { $path = "C:/wamp64/bin/apache/apache2.4.27/bin/"; }
  if (isWindows()) { $path = ""; }
  else             { $path = ""; }
  $command = "{$path}apache2 -v"; //
  $output  = _getCommandOutput($command);
  if (preg_match("|Apache/(\S+)|", $output, $matches)) { // eg: Server version: Apache/2.4.6 (CentOS)
    $version = $matches[1];
    $versionList .= "$version (Source: $command)\n";
  }

  // apachectl -v
  $version = ''; 
  //if (isWindows()) { $path = "C:/wamp64/bin/apache/apache2.4.27/bin/"; }
  if (isWindows()) { $path = ""; }
  else             { $path = ""; }
  $command = "{$path}apachectl -v"; //
  $output  = _getCommandOutput($command);
  if (preg_match("|Apache/(\S+)|", $output, $matches)) { // eg: Server version: Apache/2.4.6 (CentOS)
    $version = $matches[1];
    $versionList .= "$version (Source: $command)\n";
  }
  
  // rpm -qi httpd
  if (!isWindows()) {
    $version = ''; 
    $command = "rpm -qi httpd"; //
    $output  = _getCommandOutput($command);
    if (preg_match("|Version\s*:\s*(\S+)|", $output, $matches)) { // eg: Version     : 2.4.6
      $version = $matches[1];
      $versionList .= "$version (Source: $command)\n";
    }
  }
  
  // yum info installed httpd
  if (!isWindows()) {
    $version = ''; 
    $command = "yum info installed httpd"; //
    $output  = _getCommandOutput($command);
    if (preg_match("|Version\s*:\s*(\S+)|", $output, $matches)) { // eg: Version     : 2.4.6
      $version = $matches[1];
      $versionList .= "$version (Source: $command)\n";
    }
  }
  
  // dpkg --status apache2
  if (!isWindows()) {
    $version = ''; 
    $command = "dpkg --status apache2"; //
    $output  = _getCommandOutput($command);
    if (preg_match("|Version\s*:\s*([a-z\-\d\.\+]+)|", $output, $matches)) { // eg: Version: 2.4.29-1ubuntu4.6
      $version = $matches[1];
      $versionList .= "$version (Source: $command)\n";
    }
  }

  return $versionList;
}

//
function _getApacheVersion_fromPHP() {
  $versionList = ''; 
    
  // $_SERVER['SERVER_SOFTWARE']
  $output = $_SERVER['SERVER_SOFTWARE'];
  _debugOutput("Checking \$_SERVER['SERVER_SOFTWARE'] value:\n" .$_SERVER['SERVER_SOFTWARE']. "\n");
  if (preg_match("|Apache/(\S+)|", $output, $matches)) { // eg: Apache/2.4.27 (Win64) OpenSSL/1.1.0f PHP/7.3.2
    $version = $matches[1];
    $versionList .= "$version (Source: \$_SERVER['SERVER_SOFTWARE'])\n";
  }

  // $_SERVER['SERVER_SIGNATURE']
  $output = $_SERVER['SERVER_SIGNATURE'];
  _debugOutput("Checking \$_SERVER['SERVER_SIGNATURE'] value:\n" .$_SERVER['SERVER_SIGNATURE']. "\n");
  if (preg_match("|Apache/(\S+)|", $output, $matches)) { // eg: Apache/2.4.27 (Win64) OpenSSL/1.1.0f PHP/7.3.2
    $version = $matches[1];
    $versionList .= "$version (Source: \$_SERVER['SERVER_SIGNATURE'])\n";
  }

  // apache_get_version();
  if (function_exists('apache_get_version')) {
    $output = apache_get_version();
    _debugOutput("Checking apache_get_version() value:\n$output\n");
    if (preg_match("|Apache/(\S+)|", $output, $matches)) { // eg: <address>Apache/2.4.27 (Win64) OpenSSL/1.1.0f PHP/7.3.2 Server at localhost Port 80</address>
      $version = $matches[1];
      $versionList .= "$version (Source: apache_get_version())\n";
    }
  }

  // $_SERVER['SSL_VERSION_INTERFACE'] // Only set on https connections
  if (isset($_SERVER['SSL_VERSION_INTERFACE'])) { 
    $output = $_SERVER['SSL_VERSION_INTERFACE'];
    _debugOutput("Checking \$_SERVER['SSL_VERSION_INTERFACE'] [Apache/mod_ssl]) value:\n$output\n");
    if (preg_match("|mod_ssl/(\S+)|", $output, $matches)) { // eg: mod_ssl/2.4.27
      $version = $matches[1];
      $versionList .= "$version (Source: \$_SERVER['SSL_VERSION_INTERFACE'] [Apache/mod_ssl])\n";
    }
  }
 
  return $versionList;  
}



//
function _getApacheVersion_fromOther() {
  $versionList = ""; 
    
  // apache in process list (dev) - this is here for debugging
  if (!isWindows()) { 
    $command = 'ps -o pid,user,command | grep -e "apache" -e "http" | grep -v "grep"'; // 
    $output  = _getCommandOutput($command);
    //if (preg_match("|mod_ssl/(\S+)|", $output, $matches)) { // eg: mod_ssl/2.4.27
    //  $version = $matches[1];
    //  print "$version (Source: \$_SERVER['SSL_VERSION_INTERFACE'] [Apache/mod_ssl])\n";
    //}
  }
  
  return $versionList;
}

//
function _getCommandOutput($command) {
  $debugOutput = '';

  //
  static $firstRun = true;
  if ($firstRun) {
    if (!isWindows()) {
      
      // linux - make sure common dirs are in PATH env variable so they'll be searched for binaries
      $oldPath = isset($_SERVER['PATH']) ? $_SERVER['PATH'] : '';
      $dirs    = explode(":", $oldPath);
      array_push($dirs, '/usr/local/sbin', '/usr/local/bin', '/usr/sbin', '/usr/bin', '/bin', '/usr/local/apache/bin/');
      $newPath = implode(':', array_unique($dirs)); // remove duplicates without changing order
      putenv("PATH=$newPath");
      
    }
    $firstRun = false;
  }

  // skip if exec() disabled
  if (!function_exists('exec')) {
    _debugOutput("Skipping command: $command\nPHP exec() function disabled with 'disabled_functions' directive.\n\n");
    return null;
  }

  //
  $command .= " 2>&1"; // get stderr in output  
  exec($command, $outputArray, $exitCode);
  $output = implode("\n", $outputArray);
  _debugOutput("Executing: $command\nExit Code: $exitCode\n$output\n\n");
  
  // only return if exit code indicates success
  if ($exitCode == 0) { return $output; } // Windows: https://www.symantec.com/connect/articles/windows-system-error-codes-exit-codes-description
  else                { return null; } 
}

// Return debug output, pass argumne to append to debug output.
function _debugOutput($append = null) {

  static $debugOutput;
  if (!$debugOutput) {
    $debugOutput .= "Domain: {$_SERVER['HTTP_HOST']}\n"; 
    $debugOutput .= "PHP Version: " .phpversion(). "\n";
    $debugOutput .= " - php_sapi_name: " .php_sapi_name(). "\n";
    $debugOutput .= " - disable_functions: " .ini_get('disable_functions'). "\n";
    $debugOutput .= " - open_basedir: " .ini_get('open_basedir'). "\n";
    $debugOutput .= "\n";
  }

  // append debug info
  if (!is_null($append)) { $debugOutput .= htmlspecialchars($append) . "\n"; } 

  //
  $debugOutput = preg_replace("/\n{3,}/", "\n\n", $debugOutput); 
  return $debugOutput;
}

//
function isWindows() {
  return PHP_OS == 'WINNT';
}

// eof